#include <minimisc.h>
#include <cdb.h>
#include "lxmisc.h"
#include "misc.h"
#include "naw_log.h"


/**************************************************************************
  module_call

  call a module from the module hash
  if the module does not exist in the module hash, it will be loaded and
  added first.  If the module hash is not allocated, it will be renewed.

  Note that [name] must be null-terminated, though [namelen] must equal the
  length of [name] minus the terminating 0.

  XXX: need to get naw_handle in here somewhere
  XXX: need a way to differentiate between accept() and connect()

  returns the value of the relevant module function
  (should be NAW_YES for acceptance, NAW_NO for rejection).
 **************************************************************************/

int module_call (c, name, namelen, handle, data)
    struct cdb *c;
    char *name;
    unsigned int namelen;
    struct naw_handle *handle;
    struct naw_data *data;
{
    struct naw_module *mod;
    static blobset *bs = 0;

    if (!module_check_flush() && bs) {
        NAW_LOG(NAW_DEBUG3, LXM_q("flushing modules"));
        module_reset_flush_counter();
        blob_destroy(bs);
        bs = 0;
    }
    
    if (!bs) {
        bs = blob_new(20, 0);
        if (!bs) return NAW_ERR;

        if (modentry_globals(c)) {
            blob_destroy(bs);
            bs=0;
            return NAW_ERR;
        }
    }

    mod = blob_get (bs, name, namelen);
    if (!mod) {
        mod = naw_module_new (name, data);
        if (!mod) return NAW_ERR;
        if (blob_register(bs, name, namelen, mod, naw_module_free))
            return NAW_ERR;
    }

    switch (handle->type) {
        case INIT_SRV: return mod->f->accept (handle, data);
        case INIT_CLI: return mod->f->connect(handle, data);
        default:       return NAW_OK;
    }
}
