#include "lxmisc.h"
#include "misc.h"
#include "data.h"

#define BANNER_SUPPORT_PREFIX "NAW supported protocols: "

char naw_protocols[] = { 0x01, 0 };

void send_banner (gd, dat, okay)
	lx_gd *gd;
	cfg_data *dat;
	unsigned char *okay;
{
	*okay = 0;

	/* XXX: in the future, add protocol selection support */

	if (lx_gdputs (gd, BANNER_SUPPORT_PREFIX)
	 || lx_gdputs (gd, naw_protocols)
	 || lx_gdputc (gd, 0)
	 || lx_gdflush (gd)) return;
	else (*okay)++;
}

#if 0
	get_proto_from_banner()
	used by the client, to read the banner string from the server,
	and choose a suitable protocol.
#endif

void get_proto_from_banner (gd, dat, proto, okay)
	lx_gd *gd;
	cfg_data *dat;
	unsigned char *proto, *okay;
{
	unsigned int prefix_len = sizeof(BANNER_SUPPORT_PREFIX-1);
	lx_s ns = {0};
	lx_s ps;       /* proto (mirrored) */
	char match;

	*okay = 0;

	/* XXX add hooks for protocol selection support */

	/* 255 protocols + \0 == 256 */
	if (lx_getseg (&ns, gd, "\0", &match, prefix_len + 256)) {
		log_debug (1, "get_proto_from_banner(): couldn't get 0-delimited segment", 0,0,0,0);
		return;
	}

	if (match & MATCH_TOOLONG) {
		log_debug (1, "get_proto_from_banner(): protocol string too long", 0,0,0,0);
		goto RETURN;
	}

	if (!(match & MATCH_OK)) {
		log_debug (1, "get_proto_from_banner(): binary zero expected", 0,0,0,0);
		goto RETURN;
	}

	lx_mirror (&ps, &ns);

	ps.s     += prefix_len;
	ps.alloc -= prefix_len;
	ps.len   -= prefix_len;

	match = 0;

	/* find a suitable protocol */
	for (;;) {
		if (!naw_protocols[(int)match]) {
			match = 0;
			ps.s++;
			ps.alloc--;
			ps.len--;
		}
		if (!ps.len) break;

		if (*(ps.s) == naw_protocols[(int)match]) {
			*proto = naw_protocols[(int)match];
			(*okay)++;
			break;
		}

		match++;
	}

RETURN:
	lx_free (&ns);
}
