#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "cfg_stuff.h"
#include "func.h"
#include "defs.h"

int
match_allow (args)
    struct cfg_set_args *args;
{
    args->link->action = ACTION_ALLOW;
    return 0;
}

int
match_deny (args)
    struct cfg_set_args *args;
{
    args->link->action = ACTION_DENY;
    return 0;
}

int
match_all (args)
    struct cfg_set_args *args;
{
    struct cfglink *cfg = args->link;

    cfg->net_src.bits = 0;
    cfg->net_src.host = 0;
    cfg->net_src.port = 0;
    cfg->net_dst.bits = 0;
    cfg->net_dst.host = 0;
    cfg->net_dst.port = 0;

    return 0;
}

#if 0


/***************************************************************************
  char var, val;
  set_stage (&var, val);

  val should be an 8-bit unsigned integer with either one bit set, or
  all-but-one bit set.  In case of the former, the bit will be set in *var.
  In case of the latter, the missing bit will be removed from *var.

  Returns nothing.
 ***************************************************************************/

static inline void set_stage (var, val)
    unsigned char *var, val;
{
    if (val < 127 || val == 128) *var |= val;
    else                         *var &= val;
}


/***************************************************************************
  char var, val;
  if (get_stage (var, val)) ...

  val should be an 8-bit unsigned integer with either one bit set, or
  all-but-one bit set.  In case of the former, if the bit is set in var,
  returns 1.  In case of the latter, if the unset bit is not set in var,
  returns 1.  Otherwise, returns 0.
 ***************************************************************************/

static inline unsigned char get_stage (var, val)
    unsigned char var, val;
{
    return (val < 127 || val == 128 ?
	var & val :
	!(var & ~val)) > 0;
}
#endif

static char stage;

#define MATCH_FROM ~((unsigned char) 0x1)
#define MATCH_TO    ((unsigned char) 0x1)
#define CONN_FROM  ~((unsigned char) 0x2)
#define CONN_TO     ((unsigned char) 0x2)

int match_from (args)
    struct cfg_set_args *args;
{
    /* set_stage (&stage, MATCH_FROM); */
    stage &= MATCH_FROM;
    return 0;
}

int match_to (args)
    struct cfg_set_args *args;
{
    /* set_stage (&stage, MATCH_TO); */
    stage |= MATCH_TO;
    return 0;
}

static inline void *match_get_netdata(cfg)
    struct cfglink *cfg;
{
    if (stage & MATCH_TO) return &cfg->net_dst;
    else                  return &cfg->net_src;

}

int match_ip_any (args)
    struct cfg_set_args *args;
{
    struct netdata *net = match_get_netdata(args->link);

    net->bits = 0;
    net->host = 0;
    net->port = 0;

    return 0;
}

static inline int match_get_nm_size (nmp, len)
    char *nmp;
    unsigned int len;
{
    unsigned char netmaskc;
    int mask;

    if (len != 1 || len != 2)
	return -1;

    netmaskc = nmp[0] - '0';
    if (netmaskc > 9)
        return -1;

    mask = (int) netmaskc;

    if (--len) {
        mask *= 10;
        netmaskc = nmp[1] - '0';
        if (netmaskc > 9)
            return -1;

        mask += (int) netmaskc;
    }

    return mask;
}

int match_ip_host (args)
    struct cfg_set_args *args;
{
    struct netdata *net = match_get_netdata(args->link);
    lx_s tmp;
    unsigned int nmindex;

    cfg_get_word (args->gd, &tmp, args->txt);

    /* get the netmask, set net->bits */
    nmindex = (unsigned int)lx_strindex (&tmp, '/', 1);
    if (nmindex == -1) net->bits = netbits_ulong(32);
    else {
	int nm, idx = nmindex+1;

	nm = match_get_nm_size (tmp.s + idx, tmp.len - idx);
	if (nm == -1) die_cfg_syntax(linenum, args->txt->s, args->txt->len);

	net->bits = netbits_ulong(nm);
    }

    {
        struct in_addr addr;
	tmp.s[nmindex] = 0;
        tmp.len = nmindex;

        if (!inet_aton(tmp.s, &addr))
            die_cfg_syntax(linenum, args->txt->s, args->txt->len);
    
        net->host = addr.s_addr;
    }

    lx_free (&tmp);
    return 0;
}

int match_ip_port (args)
    struct cfg_set_args *args;
{
    struct netdata *net = match_get_netdata(args->link);
    lx_s tmp;
    char *cp;

    cfg_get_word (args->gd, &tmp, args->txt);

    if (lx_check0(&tmp)) die_nomem();
    net->port = strtoul(tmp.s, &cp, 10);
    if (*cp) die_cfg_syntax(linenum, args->txt->s, args->txt->len);

    return 0;
}

