#include <stdlib.h>
#include "moduledata.h"


/********************************************************************
 * mod_link_new():
 * returns a (void) pointer to a new modlink structure.
 * returned structure is not attached.
 * returns 0 on out-of-memory error.
 ********************************************************************/
void *mod_link_new (void) {
	naw_blob_link *link = malloc (sizeof(naw_blob_link));
	if (!link) { return 0; }

	link->next = 0;

	return link;
}


/********************************************************************
 * mod_link_attach(void *registry, void *link):
 * attaches modlink structure 'link' to the registry.
 * always successful.
 * returns the attached link (which is the same as the passed 'link')
 ********************************************************************/
void *mod_link_attach (void *registry, void *link) {
	naw_blob_link *thislink = (naw_blob_link *)link;
	naw_blob_link *testlink = registry;

	while (testlink->next)
		testlink = testlink->next;

	testlink->next = thislink;

	return thislink;
}


/********************************************************************
 * mod_link_detach(void **registry, void *link):
 * detaches modlink structure 'link' from the registry.
 * if *registry changes (i.e. the first link was matched), the new
 * registry location is placed in *registry.
 * if there is no such link in the registry, returns 0
 * else, returns the detached link.
 ********************************************************************/
void *mod_link_detach (void **registry, void *link) {
	naw_blob_link *thislink = (naw_blob_link *)link;

	if (*registry == thislink) *registry = thislink->next;
	else {
		naw_blob_link *testlink = *registry;
		while (testlink->next != thislink) {
			if (!testlink->next) return 0;
			testlink = testlink->next;
		}

		testlink->next = thislink->next;
	}

	thislink->next = 0;
	return link;
}


/********************************************************************
 * mod_link_destroy(void *link):
 * destroys the passed link, freeing all allocated data.
 * returns nothing.
 ********************************************************************/
void mod_link_destroy (void *link) {
	naw_blob_link *thislink = (naw_blob_link *)link;

	free (thislink);
}


/********************************************************************
 * mod_link_add(void *registry):
 * creates a new link with mod_link_new(), then attaches it to the
 * registry with mod_link_attach().
 * returns the newly attached link, or 0 on out-of-memory error.
 ********************************************************************/
void *mod_link_add (void *registry) {
	void *link = mod_link_new();
	if (!link) return 0;

	return mod_link_attach (registry, link);
}


/********************************************************************
 * mod_link_remove(void **registry, void *link):
 * detaches 'link' from the registry with mod_link_detach(), and
 * destroys it with mod_link_destroy().
 * returns a pointer to the now invalid memory space of 'link', or 0
 * if 'link' was not found in the registry.
 ********************************************************************/
void *mod_link_remove (void **registry, void *link) {

	if (!mod_link_detach (registry, link)) return 0;
	mod_link_destroy (link);
	return link;
}
