#include <string.h>
#include "modulefunc.h"
#include "moduledata.h"
#include "conf.h"

/********************************************************************
  naw_mod_hash_new (void)

  set up main module hash (array for splitting data to optimize
  searches).  All data is set to zero.

  returns a pointer to this allocated space, or 0 on EOM error.
 ********************************************************************/
void *naw_hash_new (void) {
	return (calloc (sizeof(void *), CONFIG_MODSIZE));
}


/********************************************************************
  naw_add_hashlink

  add a new link to the hash.


  returns a pointer to the new link, or 0 on out-of-memory error.
 ********************************************************************/
void *naw_add_hashlink (void *hash, void *item, char *name) {
	int ssum = naw_strsum (name);
	naw_mod_mainlink **hashlist = hash;
	naw_mod_mainlink *link = hashlist[ssum];

	if (link) {	
		if (naw_find_link (link, name, &link)) {
			/* found link now in link */
			return link;
		} else {
			/* last link now in link */
			link = link->next = mod_link_new ();
			if (!link) return 0;

			link->name = strdup (name);
			link->label = 0;
		}
	} else {
		link = mod_link_new ();
		if (!link) return 0;

		link->name = strdup (name);
		link->label = 0;

		hashlist[ssum] = link;
	}

	return link;
}
